/*
 * jNPad v0.3 - jNPad's an Simple Text Editor written in Java
 *
 * Copyright (C) 2014-2017  rgs
 *
 * Require JDK 1.6 (or later)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Info, Questions, Suggestions & Bugs Report to rgsevero@gmail.com
 */

package jnpad.action;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.util.logging.Level;
import java.util.logging.Logger;

import jnpad.JNPadFrame;
import jnpad.config.Accelerators;
import jnpad.ui.status.StatusDisplayable.StatusType;

/**
 * The Class AlignStringsByClipboardAction.
 *
 * @version 0.3
 * @since jNPad 0.1
 */
public final class AlignStringsByClipboardAction extends AlignStringsAction {
  /** Logger */
  private static final Logger LOGGER           = Logger.getLogger(AlignStringsByClipboardAction.class.getName());

  /** UID */
  private static final long   serialVersionUID = -485124264187870371L;

  /**
   * Instantiates a new <code>AlignStringsByClipboardAction</code>.
   *
   * @param jNPad the jNPad's frame
   */
  public AlignStringsByClipboardAction(JNPadFrame jNPad) {
    super(jNPad, JNPadActions.ACTION_NAME_ALIGN_STRINGS_BY_CLIPBOARD, Accelerators.ALIGN_STRINGS_BY_CLIPBOARD);
  }

  /**
   * Perform action.
   *
   * @see jnpad.action.JNPadAction#performAction()
   */
  @Override
  public void performAction() {
    Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    Transferable transferable = clipboard.getContents(this);
    if (transferable != null) {
      try {
        String delimiter = (String) transferable.getTransferData(DataFlavor.stringFlavor);
        alignStrings(delimiter);
      }
      catch (Exception ex) {
        LOGGER.log(Level.WARNING, ex.getMessage(), ex);
        jNPad.setStatus(StatusType.ERROR, "Error - ".concat(getLabel())); //$NON-NLS-1$
      }
    }
  }

}
