/*
 * jNPad v0.3 - jNPad's an Simple Text Editor written in Java
 *
 * Copyright (C) 2014-2017  rgs
 *
 * Require JDK 1.6 (or later)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Info, Questions, Suggestions & Bugs Report to rgsevero@gmail.com
 */

package jnpad.action;

import java.awt.event.ItemEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jnpad.JNPadFrame;
import jnpad.action.JNPadActions.Group;
import jnpad.config.Config;
import jnpad.text.JNPadTextArea;
import jnpad.util.Utilities;

/**
 * The Class SelectTabSizeAction.
 *
 * @version 0.3
 * @since   jNPad v0.1
 */
public final class SelectTabSizeAction extends JNPadAction {
  /** The size. */
  private int                             size;

  /** The Constant NAME. */
  static final String                     NAME             = "select-tab-size";                 //$NON-NLS-1$

  /** The map. */
  private static Map<String, JNPadAction> map              = new HashMap<String, JNPadAction>();

  /** UID */
  private static final long               serialVersionUID = 3933157542674383675L;

  /**
   * Instantiates a new select tab size action.
   *
   * @param jNPad the jNPad's frame
   * @param size the size
   * @param selected the selected
   */
  public SelectTabSizeAction(JNPadFrame jNPad, int size, boolean selected) {
    super(jNPad, NAME, Group.CONFIGURE);
    this.size = size;

    final String name  = NAME + Utilities.SPACE_STRING + size;
    setName(name);

    setLabel(ActionBundle.getString(NAME.concat(".label"), size)); //$NON-NLS-1$
    setMnemonic(String.valueOf(size).charAt(0));
    setDescription(ActionBundle.getString(NAME.concat(".description"), size)); //$NON-NLS-1$

    setSelected(selected);
    setStateAction(true);

    map.put(name, this);
  }

  /**
   * Handle state changed.
   *
   * @param e the item event
   * @see jnpad.action.JNPadAction#handleStateChanged(java.awt.event.ItemEvent)
   */
  @Override
  public void handleStateChanged(final ItemEvent e) {
    if (ItemEvent.SELECTED == e.getStateChange()) {
      Config.TEXT_TAB_SIZE.setValue(size);
      for (JNPadTextArea etxarea : jNPad.getViewer().getGlobalTextAreas()) {
        etxarea.setTabSize(size);
      }
    }
  }

  /**
   * Gets the completions for prefix.
   *
   * @param prefix the prefix
   * @return the completions for prefix
   */
  static List<String> getCompletionsForPrefix(String prefix) {
    List<String> list = new ArrayList<String>();
    for (String key : map.keySet()) {
      JNPadAction action = map.get(key);
      if (key.startsWith(prefix) && action.isEnabled()) {
        list.add(key);
      }
    }
    Collections.sort(list);
    return list;
  }
  
  /**
   * Perform action.
   *
   * @param name the name
   * @return true, if successful
   */
  static boolean performAction(String name) {
    JNPadAction action = map.get(name);
    if (action != null && action.isEnabled()) {
      action.setSelected(!action.isSelected());
      return true;
    }
    return false;
  }
  
}
